@@ -5,8 +5,18 @@ use warnings;
 
 sub TIEHASH {
         my ($class, $obj) = @_;
-        my $self = $obj || {};
-        bless $self, $class;
+        my $self = bless $obj || {}, $class;
+	$self->compile();
+	return $self;
+}
+
+sub compile {
+	my ($self) = @_;
+	$self->{keys} = scalar keys %{$self->{properties}};
+	$self->{sort_keys} = [sort {
+		$self->{properties}->{$a}->{index} <=> $self->{properties}->{$b}->{index}
+	} grep { $self->{properties}->{$_}->{enumerable} } keys %{$self->{properties}}];
+	return $self;
 }
  
 sub STORE {
@@ -29,8 +39,10 @@ sub STORE {
                         value => $value,
 			writable => 1,
 			configurable => 1,
-			enumerable => 1
+			enumerable => 1,
+			index => ++$self->{keys}
                 };
+		push @{$self->{sort_keys}}, $key;
         } else {
 		die "Object ($self->{name}) is locked you cannot extend with new properties";
 	}
@@ -44,19 +56,11 @@ sub FETCH {
 }
  
 sub FIRSTKEY {
-        my ($key, $value) = each %{$_[0]->{properties}};
-	if ($key && !$_[0]->{properties}->{$key}->{enumerable}) {
-		return &NEXTKEY;
-	}
-	return $key;
+	goto &NEXTKEY;
 }
  
-sub NEXTKEY { 
-        my ($key, $value) = each %{$_[0]->{properties}};
-	if ($key && !$_[0]->{properties}->{$key}->{enumerable}) {
-		return &NEXTKEY;
-	}
-	return $key;
+sub NEXTKEY {
+	return (each @{$_[0]->{sort_keys}})[1];
 }
  
 sub EXISTS {
@@ -65,7 +69,9 @@ sub EXISTS {
  
 sub DELETE { 
         my $k = $_[0]->{properties}->{$_[1]};
-        !$_[0]->{locked} && $k->{writeable} ? delete $_[0]->{properties}->{$_[1]} : undef;
+        my $del = !$_[0]->{locked} && $k->{writeable} ? delete $_[0]->{properties}->{$_[1]} : undef;
+	$_[0]->compile() if $del;
+	return $del;
 }
  
 sub CLEAR {
