@@ -2,13 +2,13 @@ package One;
 
 use Rope;
 
-prototyped {
+prototyped (
 	one => sub {
 		$_[1] ? $_[1] : $_[0]->{two}();	
 	},
 	two => sub {
 		return 50;
 	}
-};
+);
 
 1;
